UNIT UCaptureForm;

INTERFACE

USES
  Windows, Messages, SysUtils, Variants, Classes, Graphics, Controls, Forms,
  Dialogs, StdCtrls, ComCtrls, xmldom, XMLIntf, msxmldom, XMLDoc,
  OleCtrls, SHDocVw,
  IdBaseComponent, IdComponent, IdTCPConnection, IdTCPClient, IdHTTP,
  ExtCtrls, IdAntiFreezeBase, IdAntiFreeze, ActiveX, MSHTML, StrUtils;

TYPE
  TCaptureThread = CLASS;
  TCaptureForm = CLASS(TForm)
    Bevel: TBevel;
    ImageGroupBox: TGroupBox;
    XMLGroupBox: TGroupBox;
    StatusBar: TStatusBar;
    Image: TImage;
    WebBrowser: TWebBrowser;

    Procedure FormCreate(Sender: TObject);
    Procedure Capture(URL: String);

    Procedure Parse(ContentType: String; Source: TMemoryStream);

    Procedure ParseXML(Source: TMemoryStream);
    Procedure ParseVFW(Source: TMemoryStream);
    Procedure ParseJPEG(Source: TMemoryStream);
    Procedure ParseMultiPart(Source: TMemoryStream);
    Procedure UpdateByXML();
    Procedure FormDestroy(Sender: TObject);
    Procedure FormClose(Sender: TObject; Var Action: TCloseAction);

  PRIVATE
    { Private declarations }
    CaptureThread: TCaptureThread;
    CaptureURL: String;
    TempFileName: String;
    ContentType: String;

    Source, XMLStream: TMemoryStream;
    TmpStream: THandleStream;

  PUBLIC
    { Public declarations }
  End;

  TCaptureThread = CLASS(TThread)
  PRIVATE
    Parent: TCaptureForm;
    Procedure VCLUpdate;
  PROTECTED
    Procedure Execute; OVERRIDE;

  PUBLIC
    CONSTRUCTOR Create(Parent: TCaptureForm);
  End;

IMPLEMENTATION

{$R *.dfm}

USES JPEG, IdURI, vfw, UMainForm;

Procedure TCaptureForm.Capture(URL: String);
Begin
  CaptureURL := URL;
  If (CaptureThread = Nil) Then
    CaptureThread := TCaptureThread.Create(self);
  CaptureThread.Resume;
End;

Procedure TCaptureForm.FormCreate(Sender: TObject);
Var
 h : Integer;
 s : String;
Begin
  CaptureURL := '';
  WebBrowser.Navigate( 'about:blank' );
  If (Length(TempFileName) > 0) Then Exit;
  TempFileName := ExtractFilePath(Paramstr(0)) + '_Temp.xml';

  s := TempFileName + #0;
  h := CreateFile(@s[1], GENERIC_WRITE or GENERIC_READ, FILE_SHARE_READ, Nil, CREATE_ALWAYS, FILE_ATTRIBUTE_TEMPORARY, FILE_FLAG_DELETE_ON_CLOSE);
  TmpStream := THandleStream.Create( h );

  ImageGroupBox.DoubleBuffered := True;
  XMLGroupBox.DoubleBuffered := True;
  XMLStream := TMemoryStream.Create;
End;

Procedure TCaptureForm.ParseJPEG(Source: TMemoryStream);
Var
  JPEGImage         : TJPEGImage;

Begin
  JPEGImage := TJPEGImage.Create;
  Source.Seek(0, soFromBeginning);
  JPEGImage.LoadFromStream( Source );
  Image.Picture.Bitmap.Dormant;

   With Image.Picture.Bitmap Do
     Begin
       PixelFormat := pf24bit;
       Height := JPEGImage.Height;
       Width := JPEGImage.Width;
       Canvas.Draw(0, 0, JPEGImage);
     End;

  JPEGImage.Free;
End;

Procedure TCaptureForm.ParseMultiPart(Source: TMemoryStream);
Var
  S, ContentType    : String;
  i, j              : LongInt;
  Stream            : TMemoryStream;
Begin
  SetString(s, PChar(Source.Memory), Source.Size);
  Stream := TMemoryStream.Create();
  i := 1;
  While True Do
    Begin
      i := PosEx('Content-Type:', S, i);  If (i = 0) Then Break;
      Inc(i, Length('Content-Type:'));
      j := i; While (j <= Source.Size) and Not (S[j] in [#10,#13]) Do Inc(j);
      ContentType := Copy(S, i, j - i);
      i := PosEx('Content-Length:', S, j); If (i = 0) Then Break;
      Inc(i, Length('Content-Length:')); j := 0;
      While (i <= Source.Size) and Not (S[i] in [#10,#13]) Do
        Begin
          If (S[i] in ['0'..'9']) Then j := j * 10 + Ord(S[i]) - Ord('0');
          Inc(i);
        End;
      While (i <= Source.Size) and (S[i] in [#10,#13]) Do Inc(i);

      Stream.SetSize(j);
      Move(PChar(@S[i])^, Stream.Memory^, j);
      Parse(ContentType, Stream);
      Inc(i, j);
    End;
    Stream.Free;
End;

Procedure TCaptureForm.ParseVFW(Source: TMemoryStream);
Var
  FmtIn             : PBitmapInfo;
  PData             : PByteArray;
  Handle            : Cardinal;
  FmtOut            : PBitmapInfoHeader;
  FileHeader        : TBitmapFileHeader;
  Buffer            : TMemoryStream;

Begin
  FmtIn := Source.Memory;
  PData := Source.Memory; PData := @PData[FmtIn.bmiHeader.biSize];

  Handle := ICImageDecompress(0, 0, FmtIn, PData, NIL);
  If (Handle = 0) Then Exit;

  FmtOut := GlobalLock(Handle);
  Buffer := TMemoryStream.Create;

  FillChar(FileHeader, SizeOf(TBitmapFileHeader), 0);
  FileHeader.bfType := $4D42;
  FileHeader.bfOffBits := SizeOf(TBitmapFileHeader) + FmtOut.biSize;
  FmtOut.biHeight := FmtOut.biHeight * -1;

  Buffer.WriteBuffer(FileHeader, SizeOf(TBitmapFileHeader));
  Buffer.WriteBuffer(FmtOut^, GlobalSize(Handle));
  GlobalUnLock(Handle);
  GlobalFree(Handle);

  Buffer.Seek(0, soFromBeginning);
  Image.Picture.Bitmap.Dormant;
  Image.Picture.Bitmap.LoadFromStream( Buffer );

  Buffer.Free;
End;

Procedure TCaptureForm.ParseXML(Source: TMemoryStream);
{
Var
 StreamAdapter: TStreamAdapter;
 Stream: IStream;
 PSI: IPersistStreamInit;
 s: String;
 ms: TMemoryStream;
 i: Integer;
 p: PChar;
 }
Begin
  XMLStream.LoadFromStream( Source );
  Source.Seek(0, soBeginning);
  TmpStream.Seek(0, soBeginning);
  SetEndOfFile( TmpStream.Handle );
  Source.SaveToStream( TmpStream );

  WebBrowser.Navigate(TempFileName);

{
  SetString(s, PChar(Source.Memory), Source.Size);
  i := Pos('>', s);
  If (i = 0) Then Exit;
  ms := TMemoryStream.Create();
  ms.WriteBuffer(Source.Memory^, i);
  s := '<?xml-stylesheet type="text/xsl" href="stylesheet.xsl" media="screen"?>';
  ms.Write(PChar(@s[1])^, Length(s));
  p := PChar(Source.Memory);
  Inc(p, i);
  ms.WriteBuffer(p^, Source.Size - i);
  ms.Seek(0, soBeginning);

  StreamAdapter := TStreamAdapter.Create( ms, soReference );
  Stream := StreamAdapter as IStream;
  PSI := WebBrowser.Document as IPersistStreamInit;
  PSI.InitNew;
  PSI.Load( Stream );


  While WebBrowser.ReadyState < READYSTATE_INTERACTIVE Do
   Application.ProcessMessages;

   ms.Free;
   }

End;

Procedure TCaptureForm.UpdateByXML;
Var
  XML               : IXMLDocument;
  XMLNode           : IXMLNode;
  XMLRect           : IXMLNode;
  X1, Y1, X2, Y2, I : LongInt;

Begin
  Try
    XML := TXMLDocument.Create(NIL); XML.Active := True;
    XML.LoadFromStream( XMLStream );

    XMLNode := XML.DocumentElement.ChildNodes['MotionDetector'];

    For I := 0 To XMLNode.ChildNodes.Count - 1 Do
      Begin
        XMLRect := XMLNode.ChildNodes[I];
        X1 := XMLRect.Attributes['x'];
        Y2 := XMLRect.Attributes['y'];
        X2 := XMLRect.Attributes['width'] + X1;
        Y1 := XMLRect.Attributes['height'] + Y2;
        Image.Picture.Bitmap.Canvas.Pen.Color := ClRed;
        Image.Picture.Bitmap.Canvas.Pen.Width := 2;
        Image.Picture.Bitmap.Canvas.Brush.Style := bsDiagCross;
        Image.Picture.Bitmap.Canvas.Rectangle(x1, y1, x2, y2);
      End;
    XML.Active := False;
  EXCEPT
  End;
End;

Procedure TCaptureForm.Parse(ContentType: String; Source: TMemoryStream);
Begin
  If (Pos('image/', ContentType) > 0) Then ImageGroupBox.Caption := ContentType;
  If (Pos('image/jpeg', ContentType) > 0) Then
    Begin
      ParseJPEG(Source);
      Exit;
    End;
  If (Pos('text/xml', ContentType) > 0) Then
    Begin
      ParseXML(Source);
      Exit;
    End;
  If (Pos('multipart', ContentType) > 0) Then
    Begin
      ParseMultiPart(Source);
      UpdateByXML;
      Exit;
    End;
  If (Pos('image/', ContentType) > 0) Then
    Begin
      ParseVFW(Source);
      Exit;
    End;
End;

Procedure TCaptureForm.FormClose(Sender: TObject; Var Action: TCloseAction);
Begin
  If (CaptureThread <> Nil) Then
  Begin
    CaptureThread.Free;
    CaptureThread := Nil;
  End;
End;

Procedure TCaptureForm.FormDestroy(Sender: TObject);
Begin
  If (CaptureThread <> Nil) and (Not CaptureThread.Terminated) Then CaptureThread.Terminate;
  If (XMLStream <> Nil) Then XMLStream.Free;
  If (TmpStream <> Nil) Then
  Begin
    TmpStream.Free;
    DeleteFile( TempFileName );
  End;
End;

{ TSortThread }

CONSTRUCTOR TCaptureThread.Create(Parent: TCaptureForm);
Begin
  self.Parent := Parent;
  INHERITED Create(False);
End;

Procedure TCaptureThread.Execute;
Var
  HTTPText          : TMemoryStream;
  HTTP              : TidHTTP;
Begin
  While true Do
    Begin
      If Terminated Then Exit;
      With Parent Do
        Begin
          Try

            HTTPText := TMemoryStream.Create;

            HTTP := TidHTTP.Create(NIL);

            If (MainForm.ProxyServerEdit.Text <> '') Then
            Begin
               HTTP.ProxyParams.ProxyServer := MainForm.ProxyServerEdit.Text;
               HTTP.ProxyParams.ProxyPort := StrToInt(MainForm.ProxyPortEdit.Text)
            End;

            HTTP.ReadTimeout := 10000;
            HTTP.HTTPOptions := HTTP.HTTPOptions + [hoInProcessAuth];
            HTTP.Request.BasicAuthentication := True;
            HTTP.Get(CaptureURL, HTTPText);

            Source := HTTPText;
            ContentType := HTTP.Response.ContentType;

            synchronize(VCLUpdate);

            HTTP.Disconnect;
            HTTP.Free; HTTPText.Free;

          EXCEPT
            ON E: Exception Do
              Begin
                StatusBar.SimpleText := 'Connecting To server ... Failed! ' + E.Message;
                break;
              End;
          End;
        End;
      sleep(40);
    End;
End;

Procedure TCaptureThread.VCLUpdate;
Begin
  Parent.StatusBar.SimpleText := Format('Connecting To server ... Done! %d bytes recived.', [Parent.Source.Size]);
  Parent.Parse(Parent.ContentType, Parent.Source);
End;

End.

