UNIT UArchiveForm;

INTERFACE

USES
  Windows, Messages, SysUtils, Variants, Classes, Graphics, Controls, Forms,
  IdBaseComponent, IdComponent, IdTCPConnection, IdTCPClient, IdHTTP, idURI,
  Dialogs, StdCtrls, ComCtrls, xmldom, XMLIntf, msxmldom, XMLDoc, ExtCtrls,
  UCommon, JPEG;

TYPE
  PRange = ^TRange;
  TRange = RECORD
    Time : Int64;
  END;

  TArchiveCaptureThread = CLASS;
  TArchiveForm = CLASS(TForm)
    VideoFrame : TImage;
    StatusBar : TStatusBar;
    RangesGroupBox : TGroupBox;
    FromDateTimePicker : TDateTimePicker;
    Label1 : TLabel;
    Label2 : TLabel;
    ToDateTimePicker : TDateTimePicker;
    Label3 : TLabel;
    AsComboBox : TComboBox;
    BuildButton : TButton;
    Label4 : TLabel;
    TimeoutEdit : TEdit;
    ShowButton : TButton;
    RangesListView : TListView;
    PROCEDURE FormCreate(Sender : TObject);
    PROCEDURE FormClose(Sender : TObject; VAR Action : TCloseAction);
    PROCEDURE BuildButtonClick(Sender : TObject);
    PROCEDURE RangesListViewDeletion(Sender : TObject; Item : TListItem);
    PROCEDURE RangesListViewSelectItem(Sender : TObject; Item : TListItem;
      Selected : Boolean);
    PROCEDURE ShowButtonClick(Sender : TObject);
  PRIVATE
    ArchiveCaptureThread : TArchiveCaptureThread;
    PSource : PSpeclabSource;
  PUBLIC
    PROCEDURE Capture(PSource : PSpeclabSource);
    PROCEDURE ShowRanges(RangesNode : IXMLNode);
  END;

  TArchiveCaptureThread = CLASS(TThread)
  PRIVATE
    Parent : TArchiveForm;
    HTTPText : TMemoryStream;
    HTTP : TidHTTP;
    VideoURL, UserName, Password : STRING;
    StatusBarText : STRING;
    Count : LongInt;
    PROCEDURE UpdateStatusBar;
    PROCEDURE UpdateVideoFrame;
  PROTECTED
    PROCEDURE execute; OVERRIDE;
  PUBLIC
    CONSTRUCTOR Create(Parent : TArchiveForm);
  END;

VAR
  ArchiveForm       : TArchiveForm;

IMPLEMENTATION

{$R *.dfm}

USES axCtrls, DateUtils;

PROCEDURE TArchiveForm.FormCreate(Sender : TObject);
BEGIN
  FromDateTimePicker.Date := Now; ToDateTimePicker.Date := Now;
  DoubleBuffered := True; ArchiveCaptureThread := NIL; PSource := NIL;
END;

{ TArchiveCaptureThread }

CONSTRUCTOR TArchiveCaptureThread.Create(Parent : TArchiveForm);
BEGIN
  INHERITED Create(True);
  self.Parent := Parent;
END;

PROCEDURE TArchiveCaptureThread.execute;
BEGIN
  HTTP := TidHTTP.Create(NIL);
  HTTPText := TMemoryStream.Create;
  HTTP.ReadTimeout := 5000;
  HTTP.HTTPOptions := HTTP.HTTPOptions + [hoInProcessAuth];
  HTTP.Request.BasicAuthentication := True;
  HTTP.Request.Username := UserName;
  HTTP.Request.Password := Password;

  StatusBarText := 'Connecting to video source ....';
  UpdateStatusBar;

  WHILE true DO
    BEGIN
      IF Terminated THEN Exit;
      TRY
        HTTPText.Clear; HTTP.Get(VideoURL + IntToStr(Count), HTTPText);

        StatusBarText := Format('Connecting to video source .... Done! %d bytes recived. Frame %d.', [HTTPText.Size, Count]);
        IF (Pos('jpeg', HTTP.Response.ContentType) > 0) THEN
          BEGIN
            UpdateVideoFrame;
            Inc(Count);
          END
        ELSE
          BEGIN
            StatusBarText := Format('Connecting to video source .... Failed! ContentType = %s.', [HTTP.Response.ContentType]);
            UpdateStatusBar;
            sleep(250);
          END;
      EXCEPT
        ON E : Exception DO
          BEGIN
            StatusBarText := Format('Connecting to video source .... Failed! %s', [E.Message]);
            UpdateStatusBar;
            sleep(250);
          END;
      END;
      sleep(0);
    END;
  HTTP.Free; HTTPText.Free;
END;

PROCEDURE TArchiveCaptureThread.UpdateStatusBar;
BEGIN
  Parent.StatusBar.SimpleText := StatusBarText;
END;

PROCEDURE TArchiveCaptureThread.UpdateVideoFrame;
VAR
  JPEGImage         : TJPEGImage;
BEGIN
  JPEGImage := TJPEGImage.Create;
  TRY
    HTTPText.Seek(0, soFromBeginning);
    JPEGImage.LoadFromStream(HTTPText);
    Parent.VideoFrame.Picture.Assign(JPEGImage);
  FINALLY
    JPEGImage.Free;
  END;
  Parent.StatusBar.SimpleText := StatusBarText;
END;

PROCEDURE TArchiveForm.FormClose(Sender : TObject; VAR Action : TCloseAction);
BEGIN
  IF (PSource <> NIL) THEN Dispose(PSource);
  IF (ArchiveCaptureThread = NIL) THEN exit;
  IF (NOT ArchiveCaptureThread.Terminated) THEN ArchiveCaptureThread.Terminate;
  ArchiveCaptureThread := NIL;
END;

PROCEDURE TArchiveForm.Capture(PSource : PSpeclabSource);
BEGIN
  new(self.PSource);
  self.PSource^.Flag := PSource^.Flag;
  self.PSource^.Name := PSource^.Name;
  self.PSource^.Moniker := PSource^.Moniker;
  self.PSource^.HostIP := PSource^.HostIP;
  self.PSource^.HostPort := PSource^.HostPort;
  self.PSource^.UserName := PSource^.UserName;
  self.PSource^.Password := PSource^.Password;
  Caption := PSource^.Name;
END;

PROCEDURE TArchiveForm.BuildButtonClick(Sender : TObject);
VAR
  HTTPText          : TMemoryStream;
  XML               : IXMLDocument;
  HTTP              : TidHTTP;
  URL               : TidURI;
BEGIN
  BuildButton.Enabled := False; StatusBar.SimpleText := 'Building Ranges ... '; RangesListView.Clear;

  URL := TidURI.Create;
  HTTP := TidHTTP.Create(NIL);
  HTTPText := TMemoryStream.Create;

  TRY
    URL.Protocol := 'http';
    URL.Host := PSource^.HostIP;
    URL.Port := PSource^.HostPort;
    URL.Path := '/' + FixupMoniker(PSource^.Moniker);
    IF (AsComboBox.ItemIndex = 0) THEN
      URL.Params := Format('/ranges?begin=%d&end=%d&interval=3600000', [DateTimeToUnix(FromDateTimePicker.Date - 1.0) * 1000, DateTimeToUnix(ToDateTimePicker.Date + 1.0) * 1000])
    ELSE
      URL.Params := Format('/ranges?begin=%d&end=%d&interval=60000', [DateTimeToUnix(FromDateTimePicker.Date - 1.0) * 1000, DateTimeToUnix(ToDateTimePicker.Date + 1.0) * 1000]);

    HTTP.ReadTimeout := StrToInt(TimeoutEdit.Text);
    HTTP.HTTPOptions := HTTP.HTTPOptions + [hoInProcessAuth];
    HTTP.Request.BasicAuthentication := True;
    HTTP.Request.Username := PSource^.UserName;
    HTTP.Request.Password := PSource^.Password;

    HTTP.Get(URL.URI, HTTPText);

    StatusBar.SimpleText := Format('Building Ranges ... Done! %d bytes recived.', [HTTPText.Size]);

    IF (Pos('text/xml', HTTP.Response.ContentType) > 0) THEN
      BEGIN
        XML := TXMLDocument.Create(NIL);
        TRY
          XML.LoadFromStream(HTTPText); ShowRanges(XML.DocumentElement);
        EXCEPT
          ON E : Exception DO
            StatusBar.SimpleText := Format('Building Ranges ... Failed! %s', [E.Message]);
        END;
      END;
  EXCEPT
    ON E : Exception DO
      StatusBar.SimpleText := Format('Building Ranges ... Failed! %s', [E.Message]);
  END;
  URL.Free; HTTP.Free; HTTPText.Free; BuildButton.Enabled := True;
END;

PROCEDURE TArchiveForm.ShowRanges(RangesNode : IXMLNode);
VAR
  Start, Finish     : TDateTime;
  i                 : LongInt;
  PRng              : PRange;
BEGIN
  TRY
    IF (RangesNode <> NIL) THEN
      FOR i := 0 TO RangesNode.ChildNodes.Count - 1 DO
        BEGIN
          Start := UnixToDateTime(StrToInt64(RangesNode.ChildNodes[i].Attributes['begin']) DIV 1000);
          Finish := UnixToDateTime(StrToInt64(RangesNode.ChildNodes[i].Attributes['end']) DIV 1000);
          WITH RangesListView.Items.Add DO
            BEGIN
              Caption := DateTimeToStr(Start);
              SubItems.Add(DateTimeToStr(Finish));
              New(PRng); PRng^.Time := StrToInt64(RangesNode.ChildNodes[i].Attributes['begin']);
              Data := PRng;
            END;
        END;
  EXCEPT
    ON E : Exception DO
      BEGIN
        StatusBar.SimpleText := Format('Build Ranges ... Failed! %s', [E.Message]);
        RangesListView.Clear;
      END;
  END;
END;

PROCEDURE TArchiveForm.RangesListViewDeletion(Sender : TObject; Item : TListItem);
VAR
  PSource           : PRange;
BEGIN
  ShowButton.Enabled := False; PSource := Item.Data; IF (PSource <> NIL) THEN Dispose(PSource);
END;

PROCEDURE TArchiveForm.RangesListViewSelectItem(Sender : TObject; Item : TListItem; Selected : Boolean);
BEGIN
  IF (Selected) THEN ShowButton.Enabled := Item.Data <> NIL ELSE ShowButton.Enabled := false;
END;

FUNCTION GetVideoURL(PSource : PSpeclabSource; PRng : PRange) : STRING;
VAR
  URL               : TidURI;
BEGIN
  URL := TidURI.Create;
  URL.Protocol := 'http';
  URL.Host := PSource^.HostIP;
  URL.Port := PSource^.HostPort;
  URL.Path := '/' + FixupMoniker(PSource^.Moniker);
  URL.Params := '/video?jpeg&begin=' + IntToStr(PRng^.Time) + '&skip=';
  GetVideoURL := URL.URI;
  URL.Free;
END;

PROCEDURE TArchiveForm.ShowButtonClick(Sender : TObject);
VAR
  PRng              : PRange;
BEGIN
  IF (ArchiveCaptureThread <> NIL) THEN
    BEGIN
      IF (NOT ArchiveCaptureThread.Terminated) THEN ArchiveCaptureThread.Terminate;
      ArchiveCaptureThread := NIL;
    END;
  ArchiveCaptureThread := TArchiveCaptureThread.Create(self);
  PRng := RangesListView.Selected.Data; IF (PRng = NIL) THEN Exit;
  ArchiveCaptureThread.UserName := PSource^.UserName;
  ArchiveCaptureThread.Password := PSource^.Password;
  ArchiveCaptureThread.VideoURL := GetVideoURL(PSource, PRng);
  ArchiveCaptureThread.Resume;
END;

END.
