UNIT UCaptureForm;

INTERFACE

USES
  Windows, Messages, SysUtils, Variants, Classes, Graphics, Controls, Forms,
  IdBaseComponent, IdComponent, IdTCPConnection, IdTCPClient, IdHTTP, idURI,
  Dialogs, StdCtrls, ComCtrls, xmldom, XMLIntf, msxmldom, XMLDoc, ExtCtrls,
  UCommon, JPEG;

TYPE
  TVideoCaptureThread = CLASS;
  TCaptureForm = CLASS(TForm)
    VideoFrame : TImage;
    StatusBar : TStatusBar;
    PROCEDURE FormCreate(Sender : TObject);
    PROCEDURE FormClose(Sender : TObject; VAR Action : TCloseAction);
  PRIVATE
    VideoCaptureThread : TVideoCaptureThread;
  PUBLIC
    PROCEDURE Capture(PSource : PSpeclabSource);
  END;

  TVideoCaptureThread = CLASS(TThread)
  PRIVATE
    Parent : TCaptureForm;
    HTTPText : TMemoryStream;
    HTTP : TidHTTP;
    VideoURL, UserName, Password : STRING;
    StatusBarText : STRING;
    PROCEDURE UpdateStatusBar;
    PROCEDURE UpdateVideoFrame;
  PROTECTED
    PROCEDURE execute; OVERRIDE;
  PUBLIC
    CONSTRUCTOR Create(Parent : TCaptureForm);
  END;

VAR
  CaptureForm       : TCaptureForm;

IMPLEMENTATION

{$R *.dfm}

USES axCtrls;

PROCEDURE TCaptureForm.FormCreate(Sender : TObject);
BEGIN
  DoubleBuffered := True; VideoCaptureThread := TVideoCaptureThread.Create(self);
END;

{ TVideoCaptureThread }

CONSTRUCTOR TVideoCaptureThread.Create(Parent : TCaptureForm);
BEGIN
  INHERITED Create(True);
  self.Parent := Parent;
END;

PROCEDURE TVideoCaptureThread.execute;
BEGIN
  HTTP := TidHTTP.Create(NIL);
  HTTPText := TMemoryStream.Create;
  HTTP.ReadTimeout := 2500;
  HTTP.HTTPOptions := HTTP.HTTPOptions + [hoInProcessAuth];
  HTTP.Request.BasicAuthentication := True;
  HTTP.Request.Username := UserName;
  HTTP.Request.Password := Password;

  StatusBarText := 'Connecting to video source ....';
  UpdateStatusBar;

  WHILE true DO
    BEGIN
      IF Terminated THEN Exit;
      TRY
        HTTPText.Clear; HTTP.Get(VideoURL, HTTPText);

        StatusBarText := Format('Connecting to video source .... Done! %d bytes recived.', [HTTPText.Size]);

        IF (Pos('jpeg', HTTP.Response.ContentType) > 0) THEN UpdateVideoFrame
        ELSE
          BEGIN
            StatusBarText := Format('Connecting to video source .... Failed! ContentType = %s.', [HTTP.Response.ContentType]);
            UpdateStatusBar;
            sleep(250);
          END;
      EXCEPT
        ON E : Exception DO
          BEGIN
            StatusBarText := Format('Connecting to video source .... Failed! %s', [E.Message]);
            UpdateStatusBar;
            sleep(250);
          END;
      END;
      sleep(0);
    END;
  HTTP.Free; HTTPText.Free;
END;

PROCEDURE TVideoCaptureThread.UpdateStatusBar;
BEGIN
  Parent.StatusBar.SimpleText := StatusBarText;
END;

PROCEDURE TVideoCaptureThread.UpdateVideoFrame;
VAR
  JPEGImage         : TJPEGImage;
BEGIN
  JPEGImage := TJPEGImage.Create;
  TRY
    HTTPText.Seek(0, soFromBeginning);
    JPEGImage.LoadFromStream(HTTPText);
    Parent.VideoFrame.Picture.Assign(JPEGImage);
  FINALLY
    JPEGImage.Free;
  END;
  Parent.StatusBar.SimpleText := StatusBarText;
END;

PROCEDURE TCaptureForm.FormClose(Sender : TObject; VAR Action : TCloseAction);
BEGIN
  IF (NOT VideoCaptureThread.Terminated) THEN VideoCaptureThread.Terminate;
END;

FUNCTION GetVideoURL(PSource : PSpeclabSource) : STRING;
VAR
  URL               : TidURI;
BEGIN
  URL := TidURI.Create;
  URL.Protocol := 'http';
  URL.Host := PSource^.HostIP;
  URL.Port := PSource^.HostPort;
  URL.Path := '/' + FixupMoniker(PSource^.Moniker);
  URL.Params := '/video?jpeg';
  GetVideoURL := URL.URI;
  URL.Free;
END;

PROCEDURE TCaptureForm.Capture(PSource : PSpeclabSource);
BEGIN
  Caption := PSource^.Name;
  VideoCaptureThread.UserName := PSource^.UserName;
  VideoCaptureThread.Password := PSource^.Password;
  VideoCaptureThread.VideoURL := GetVideoURL(PSource);
  VideoCaptureThread.Resume;
END;

END.
