UNIT UMainForm;

INTERFACE

USES
  Windows, Messages, SysUtils, Variants, Classes, Graphics, Controls, Forms,
  IdBaseComponent, IdComponent, IdTCPConnection, IdTCPClient, IdHTTP, idURI,
  Dialogs, StdCtrls, ComCtrls, xmldom, XMLIntf, msxmldom, XMLDoc;

TYPE
  TMainForm = CLASS(TForm)
    HostGroupBox : TGroupBox;
    Label1 : TLabel;
    HostIPEdit : TEdit;
    Label2 : TLabel;
    HostPortEdit : TEdit;
    StatusBar : TStatusBar;
    StartButton : TButton;
    SourcesListView : TListView;
    Label3 : TLabel;
    LoginEdit : TEdit;
    Label4 : TLabel;
    Label5 : TLabel;
    PasswordEdit : TEdit;
    TimeoutEdit : TEdit;
    ShowButton : TButton;
    PROCEDURE StartButtonClick(Sender : TObject);
    PROCEDURE SourcesListViewDeletion(Sender : TObject; Item : TListItem);
    PROCEDURE SourcesListViewSelectItem(Sender : TObject; Item : TListItem; Selected : Boolean);
    PROCEDURE ShowButtonClick(Sender : TObject);
    procedure FormCreate(Sender: TObject);
  PRIVATE
    PROCEDURE EnumAllSources(SourceName : STRING; SourceNode : IXMLNode);
    PROCEDURE CheckSource(SourceName : STRING; SourceNode : IXMLNode);
    FUNCTION IsRealTimeVideoSource(Moniker : STRING; VAR Error : Boolean) : Boolean;
    FUNCTION IsArchiveSource(Moniker : STRING; VAR Error : Boolean) : Boolean;
  PUBLIC
  END;

VAR
  MainForm          : TMainForm;

IMPLEMENTATION

{$R *.dfm}
USES UCaptureForm, UArchiveForm, UCommon;

FUNCTION TMainForm.IsRealTimeVideoSource(Moniker : STRING; VAR Error : Boolean) : Boolean;
VAR
  HTTPText          : TMemoryStream;
  XML               : IXMLDocument;
  HTTP              : TidHTTP;
  URL               : TidURI;
  ScriptsNode       : IXMLNode;
  I                 : LongInt;
  bRanges           : Boolean;
  bVideo            : Boolean;
BEGIN
  Error := false; bRanges := false; bVideo := false;

  URL := TidURI.Create;
  HTTP := TidHTTP.Create(NIL);
  HTTPText := TMemoryStream.Create;

  TRY
    URL.Protocol := 'http';
    URL.Host := HostIPEdit.Text;
    URL.Port := HostPortEdit.Text;
    URL.Path := '/' + FixupMoniker(Moniker);
    URL.Params := '/';

    HTTP.ReadTimeout := 2500;
    HTTP.HTTPOptions := HTTP.HTTPOptions + [hoInProcessAuth];
    HTTP.Request.BasicAuthentication := True;
    HTTP.Request.Username := LoginEdit.Text;
    HTTP.Request.Password := PasswordEdit.Text;

    HTTP.Get(URL.URI, HTTPText);

    IF (Pos('text/xml', HTTP.Response.ContentType) > 0) THEN
      BEGIN
        XML := TXMLDocument.Create(NIL);
        TRY
          XML.LoadFromStream(HTTPText);
          ScriptsNode := XML.DocumentElement.ChildNodes.FindNode('scripts');
          IF (ScriptsNode <> NIL) THEN
            FOR i := 0 TO ScriptsNode.ChildNodes.Count - 1 DO
              IF (ScriptsNode.ChildNodes[i].HasAttribute('name')) THEN
                BEGIN
                  IF (ScriptsNode.ChildNodes[i].Attributes['name'] = 'video') THEN bVideo := true;
                  IF (ScriptsNode.ChildNodes[i].Attributes['name'] = 'ranges') THEN bRanges := true;
                END;
        EXCEPT
          ON E : Exception DO Error := True;
        END;
      END;
  EXCEPT
    ON E : Exception DO Error := True;
  END;
  URL.Free; HTTP.Free; HTTPText.Free;
  IsRealTimeVideoSource := (NOT Error) AND (NOT bRanges) AND (bVideo);
END;

FUNCTION TMainForm.IsArchiveSource(Moniker : STRING; VAR Error : Boolean) : Boolean;
VAR
  HTTPText          : TMemoryStream;
  XML               : IXMLDocument;
  HTTP              : TidHTTP;
  URL               : TidURI;
  ScriptsNode       : IXMLNode;
  I                 : LongInt;
  bRanges           : Boolean;
  bVideo            : Boolean;
BEGIN
  Error := false; bRanges := false; bVideo := false;

  URL := TidURI.Create;
  HTTP := TidHTTP.Create(NIL);
  HTTPText := TMemoryStream.Create;

  TRY
    URL.Protocol := 'http';
    URL.Host := HostIPEdit.Text;
    URL.Port := HostPortEdit.Text;
    URL.Path := '/' + FixupMoniker(Moniker);
    URL.Params := '/';

    HTTP.ReadTimeout := 2500;
    HTTP.HTTPOptions := HTTP.HTTPOptions + [hoInProcessAuth];
    HTTP.Request.BasicAuthentication := True;
    HTTP.Request.Username := LoginEdit.Text;
    HTTP.Request.Password := PasswordEdit.Text;

    HTTP.Get(URL.URI, HTTPText);

    IF (Pos('text/xml', HTTP.Response.ContentType) > 0) THEN
      BEGIN
        XML := TXMLDocument.Create(NIL);
        TRY
          XML.LoadFromStream(HTTPText);
          ScriptsNode := XML.DocumentElement.ChildNodes.FindNode('scripts');
          IF (ScriptsNode <> NIL) THEN
            FOR i := 0 TO ScriptsNode.ChildNodes.Count - 1 DO
              IF (ScriptsNode.ChildNodes[i].HasAttribute('name')) THEN
                BEGIN
                  IF (ScriptsNode.ChildNodes[i].Attributes['name'] = 'video') THEN bVideo := true;
                  IF (ScriptsNode.ChildNodes[i].Attributes['name'] = 'ranges') THEN bRanges := true;
                END;
        EXCEPT
          ON E : Exception DO Error := True;
        END;
      END;
  EXCEPT
    ON E : Exception DO Error := True;
  END;
  URL.Free; HTTP.Free; HTTPText.Free;
  IsArchiveSource := (NOT Error) AND (bRanges) AND (bVideo);
END;

PROCEDURE TMainForm.CheckSource(SourceName : STRING; SourceNode : IXMLNode);
VAR
  Moniker           : STRING;
  VideoSignalNode   : IXMLNode;
  PSource           : PSpeclabSource;
  Error             : Boolean;
  VideoSignal       : Boolean;
LABEL
  Found;
BEGIN
  StatusBar.SimpleText := Format('Checking "%s" ...', [SourceName]);
  Moniker := SourceNode.ChildNodes['moniker'].Text;
  WITH SourcesListView.Items.Add DO
    BEGIN
      Caption := SourceName; Data := NIL;

      VideoSignal := true;
      VideoSignalNode := SourceNode.ChildNodes.FindNode('videosignal');
      IF (VideoSignalNode <> NIL) AND (SourceNode.ChildNodes['videosignal'].IsTextElement) AND (SourceNode.ChildNodes['videosignal'].Text <> 'true') THEN VideoSignal := false;

      IF (NOT VideoSignal) THEN
        BEGIN
          SubItems.Add('No signal');
          GOTO Found;
        END;

      IF (IsArchiveSource(Moniker, Error)) THEN
        BEGIN
          SubItems.Add('Archive');
          New(PSource);
          PSource^.Flag := SL_ARCHIVE;
          PSource^.Name := SourceName;
          PSource^.Moniker := Moniker;
          PSource^.HostPort := HostPortEdit.Text;
          PSource^.HostIP := HostIPEdit.Text;
          PSource^.Username := LoginEdit.Text;
          PSource^.Password := PasswordEdit.Text;
          Data := PSource; GOTO Found;
        END;
      IF (Error) THEN
        BEGIN
          SubItems.Add('Error'); GOTO Found;
        END;

      IF (IsRealTimeVideoSource(Moniker, Error)) THEN
        BEGIN
          SubItems.Add('Realtime');
          New(PSource);
          PSource^.Flag := SL_REALTIME;
          PSource^.Name := SourceName;
          PSource^.Moniker := Moniker;
          PSource^.HostPort := HostPortEdit.Text;
          PSource^.HostIP := HostIPEdit.Text;
          PSource^.Username := LoginEdit.Text;
          PSource^.Password := PasswordEdit.Text;
          Data := PSource; GOTO Found;
        END;
      IF (Error) THEN
        BEGIN
          SubItems.Add('Error'); GOTO Found;
        END;
      SubItems.Add('Not video source');
    END;

  Found : StatusBar.SimpleText := Format('Checking "%s" ... Done.', [SourceName]);
END;

PROCEDURE TMainForm.SourcesListViewDeletion(Sender : TObject; Item : TListItem);
VAR
  PSource           : PSpeclabSource;
BEGIN
  ShowButton.Enabled := False; PSource := Item.Data; IF (PSource <> NIL) THEN Dispose(PSource);
END;

PROCEDURE TMainForm.EnumAllSources(SourceName : STRING; SourceNode : IXMLNode);
VAR
  i                 : LongInt;
BEGIN
  IF (SourceNode.NodeName = 'grandchild') THEN
    IF (SourceNode.ChildNodes.FindNode('description') <> NIL) THEN
      CheckSource(SourceName + SourceNode.ChildNodes['description'].Text, SourceNode);
  IF (SourceNode.ChildNodes.FindNode('description') <> NIL) THEN
    SourceName := SourceName + SourceNode.ChildNodes['description'].Text + '\';
  FOR i := 0 TO SourceNode.ChildNodes.Count - 1 DO
    EnumAllSources(SourceName, SourceNode.ChildNodes[i]);
END;

PROCEDURE TMainForm.StartButtonClick(Sender : TObject);
VAR
  HTTPText          : TMemoryStream;
  XML               : IXMLDocument;
  HTTP              : TidHTTP;
  URL               : TidURI;
BEGIN
  StartButton.Enabled := False; StatusBar.SimpleText := 'Connecting to server ...'; SourcesListView.Clear;

  URL := TidURI.Create;
  HTTP := TidHTTP.Create(NIL);
  HTTPText := TMemoryStream.Create;

  TRY
    URL.Protocol := 'http';
    URL.Host := HostIPEdit.Text;
    URL.Port := HostPortEdit.Text;
    URL.Path := '/enum/all';

    HTTP.ReadTimeout := StrToInt(TimeoutEdit.Text);
    HTTP.HTTPOptions := HTTP.HTTPOptions + [hoInProcessAuth];
    HTTP.Request.BasicAuthentication := True;
    HTTP.Request.Username := LoginEdit.Text;
    HTTP.Request.Password := PasswordEdit.Text;
    HTTP.Get(URL.URI, HTTPText);

    StatusBar.SimpleText := Format('Connecting to server ... Done! %d bytes recived.', [HTTPText.Size]);

    IF (Pos('text/xml', HTTP.Response.ContentType) > 0) THEN
      BEGIN
        XML := TXMLDocument.Create(NIL);
        TRY
          XML.LoadFromStream(HTTPText); EnumAllSources('', XML.DocumentElement);
        EXCEPT
          ON E : Exception DO
            StatusBar.SimpleText := Format('Connecting to server ... Failed! %s', [E.Message]);
        END;
      END;
  EXCEPT
    ON E : Exception DO
      StatusBar.SimpleText := Format('Connecting to server ... Failed! %s', [E.Message]);
  END;
  URL.Free; HTTP.Free; HTTPText.Free; StartButton.Enabled := True;
END;

PROCEDURE TMainForm.SourcesListViewSelectItem(Sender : TObject; Item : TListItem; Selected : Boolean);
BEGIN
  IF (Selected) THEN ShowButton.Enabled := Item.Data <> NIL ELSE ShowButton.Enabled := false;
END;

PROCEDURE TMainForm.ShowButtonClick(Sender : TObject);
VAR
  NewCaptureForm    : TCaptureForm;
  NewArchiveForm    : TArchiveForm;
  PSource           : PSpeclabSource;
BEGIN
  PSource := SourcesListView.Selected.Data; IF (PSource = NIL) THEN Exit;
  IF (PSource^.Flag = SL_REALTIME) THEN
    BEGIN
      NewCaptureForm := TCaptureForm.Create(self);
      NewCaptureForm.Show;
      NewCaptureForm.Capture(PSource);
      Exit;
    END;
  IF (PSource^.Flag = SL_ARCHIVE) THEN
    BEGIN
      NewArchiveForm := TArchiveForm.Create(self);
      NewArchiveForm.Show;
      NewArchiveForm.Capture(PSource);
      Exit;
    END;
END;

procedure TMainForm.FormCreate(Sender: TObject);
begin
  ShowMessage('   GoalCity    4.20.82!');
end;

END.
