{$APPTYPE CONSOLE}
USES
  SysUtils, ActiveX, Classes, IdHTTP, IdURI, xmldom, XMLIntf, msxmldom, XMLDoc, UCommon, StrUtils;

CONST
  DW_TIME_OUT       = 50000 * 3;
  STR_LOGIN         = '';
  STR_PASSWORDD     = '';

VAR
  ServerIP, ServerPort, SensorName, SensorState : STRING;

PROCEDURE StringToStream(CONST Text : STRING; CONST Stream : TMemoryStream);
BEGIN
  Stream.SetSize(Length(Text));
  Stream.Seek(0, soFromBeginning);
  Stream.Write(Text[1], Length(Text));
  Stream.Seek(0, soFromBeginning);
END;

PROCEDURE CheckSource(SourceName : STRING; SourceNode : IXMLNode);
VAR
  HTTPRequest       : TMemoryStream;
  HTTPText          : TMemoryStream;
  HTTP              : TidHTTP;
  URL               : TidURI;

  sXML              : String;
  sCurrentState     : String; //   .   
  i1, i2            : Integer;
BEGIN
  IF (UpperCase(SourceName) = UpperCase(SensorName)) THEN
    BEGIN
      Write('Connecting to sensor GET ... ');

      URL := TidURI.Create;
      HTTP := TidHTTP.Create(NIL);
      HTTPText := TMemoryStream.Create;
      HTTPRequest := TMemoryStream.Create;

      TRY
        URL.Protocol := 'http';
        URL.Host := ServerIP;
        URL.Port := ServerPort;
        URL.Path := '/' + FixupMoniker(SourceNode.ChildNodes['moniker'].Text);
        URL.Params := '/config';

        HTTP.ReadTimeout := DW_TIME_OUT;
        HTTP.HTTPOptions := HTTP.HTTPOptions + [hoInProcessAuth];
        HTTP.Request.BasicAuthentication := True;
        HTTP.Request.Username := STR_LOGIN;
        HTTP.Request.Password := STR_PASSWORDD;

        HTTP.Get(URL.URI, HTTPText);

        IF (Pos('text/xml', HTTP.Response.ContentType) > 0) THEN
          WriteLn('OK.')
        ELSE
          WriteLn('FAIL!');

      EXCEPT
        ON E : Exception DO
          WriteLn(Format('Failed! %s', [E.Message]));
      END;

      SetString(sXML, PChar(HTTPText.Memory), HTTPText.Size);
      URL.Free; HTTP.Free; HTTPText.Free; HTTPRequest.Free;

      i1 := Pos('<state type="bool"', sXML);
      If (i1 = 0) Then Exit;
      i1 := PosEx('">', sXML, i1 + 18);
      If (i1 = 0) Then Exit;
      i1 := i1 + 2;
      i2 := PosEx('<', sXML, i1);
      If (i2 = 0) Then Exit;
      sCurrentState := Copy(sXML, i1, i2 - i1);
      Delete(sXML, i1, i2 - i1);

      If (SensorState = '1') Then
        Insert('false', sXML, i1)
      Else
        Insert('true', sXML, i1);


      Write('Connecting to sensor POST ... ');

      URL := TidURI.Create;
      HTTP := TidHTTP.Create(NIL);
      HTTPText := TMemoryStream.Create;
      HTTPRequest := TMemoryStream.Create;

      StringToStream(sXML, HTTPRequest);

      TRY
        URL.Protocol := 'http';
        URL.Host := ServerIP;
        URL.Port := ServerPort;
        URL.Path := '/' + FixupMoniker(SourceNode.ChildNodes['moniker'].Text);
        URL.Params := '/config';

        HTTP.ReadTimeout := DW_TIME_OUT;
        HTTP.HTTPOptions := HTTP.HTTPOptions + [hoInProcessAuth];
        HTTP.Request.BasicAuthentication := True;
        HTTP.Request.Username := STR_LOGIN;
        HTTP.Request.Password := STR_PASSWORDD;
        HTTP.Request.ContentType := 'text/xml';
        HTTP.Request.Accept := '*/*';
        HTTP.Request.ContentType := 'text/xml';
        HTTP.Request.Connection := 'Keep-Alive';
        HTTP.Request.ContentLength := length(sXML);

        HTTP.Post(URL.URI, HTTPRequest, HTTPText);

        IF (Pos('text/xml', HTTP.Response.ContentType) > 0) THEN
          WriteLn('OK.')
        ELSE
          WriteLn('FAIL!');

      EXCEPT
        ON E : Exception DO
          WriteLn(Format('Failed! %s', [E.Message]));
      END;
      URL.Free; HTTP.Free; HTTPText.Free; HTTPRequest.Free;

    END;
END;

PROCEDURE EnumAllSources(SourceNode : IXMLNode);
VAR
  i                 : LongInt;
BEGIN
  IF (SourceNode.NodeName = 'grandchild') THEN
    IF (SourceNode.ChildNodes.FindNode('description') <> NIL) THEN
      CheckSource(SourceNode.ChildNodes['description'].Text, SourceNode);
  FOR i := 0 TO SourceNode.ChildNodes.Count - 1 DO
    EnumAllSources(SourceNode.ChildNodes[i]);
END;

PROCEDURE Connect;
VAR
  HTTPText          : TMemoryStream;
  XML               : IXMLDocument;
  HTTP              : TidHTTP;
  URL               : TidURI;
BEGIN
  Write('Connecting to server ...');

  URL := TidURI.Create;
  HTTP := TidHTTP.Create(NIL);
  HTTPText := TMemoryStream.Create;

  TRY
    URL.Protocol := 'http';
    URL.Host := ServerIP;
    URL.Port := ServerPort;
    URL.Path := '/enum/all';

    HTTP.ReadTimeout := DW_TIME_OUT;
    HTTP.HTTPOptions := HTTP.HTTPOptions + [hoInProcessAuth];
    HTTP.Request.BasicAuthentication := True;
    HTTP.Request.Username := STR_LOGIN;
    HTTP.Request.Password := STR_PASSWORDD;
    HTTP.Get(URL.URI, HTTPText);

    WriteLn(Format('Done! %d bytes recived.', [HTTPText.Size]));

    IF (Pos('text/xml', HTTP.Response.ContentType) > 0) THEN
      BEGIN
        XML := TXMLDocument.Create(NIL);
        TRY
          XML.LoadFromStream(HTTPText); EnumAllSources(XML.DocumentElement);
        EXCEPT
          ON E : Exception DO
            WriteLn(Format('Failed! %s', [E.Message]));
        END;
      END;
  EXCEPT
    ON E : Exception DO
      WriteLn(Format('Failed! %s', [E.Message]));
  END;
  URL.Free; HTTP.Free; HTTPText.Free;
END;

BEGIN
  CoInitialize(NIL);
  IF (ParamCount <> 4) THEN
    BEGIN
      WriteLn('SensUtil.exe [IP-address] [Port] [SensorName] [0/1]');
      Exit;
    END;
  ServerIP := ParamStr(1); ServerPort := ParamStr(2); SensorName := ParamStr(3); SensorState := ParamStr(4);
  IF ((SensorState <> '0') AND (SensorState <> '1')) THEN
    BEGIN
      WriteLn('SensUtil.exe [IP-address] [SensorName] [0/1]');
      Exit;
    END;
  Connect;
  CoUnInitialize();
END.
